/*-------------------------------------------------------------------*/
/* CBRAND.C       Juha Kivijrvi                                     */
/*                                                                   */
/* - Muodostaa annetusta opetusjoukosta satunnaisia koodikirjoja ja  */
/*   laskee niiden virheet                                           */
/* - Luo satunnaisen koodikirjan ja ilmoittaa sen keskivirheen       */
/*                                                                   */
/*-------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <values.h>

#define  FACTFILE "cbrand.fac"
#include "parametr.c"
#include "memctrl.h"
#include "random.h"
#include "cb.h"
#include "file.h"
#include "owntypes.h"

#if defined(__MSDOS__)
#include <conio.h>
#endif

/*--------------------------------------------------------------------*/


#define ProgName        "CBRAND"
#define VersionNumber   "version 0.28"
#define LastUpdated     "26.11.97"        /* (by     Juha Kivijrvi) */



/*============================  P R I N T  ===============================*/


void PrintInfo(void)
{
/*
  printf("%s     %s                (last modified %s)\n\n",ProgName,
                                               VersionNumber,LastUpdated);
  printf("This program generates random codebooks from given training\n");
  printf("set file and calculates the average error for each codebook.\n");
  printf("This program can also be used for generating a single random ");
  printf("codebook file.\n\n");
  printf("        Usage: CBRAND [-C] [-W] [-O] [-Scodebooksize] TSfile [outputfile]\n\n");
  printf("- If option -C is given, program will create single random codebook file\n");
  printf("  and calculate its mean error. Otherwise random codebooks are generated\n");
  printf("  and evaluated until escape is pressed.\n");
  printf("- If option -W is given, program will create codebook from all the vectors\n");
  printf("  in space, not only the training set vectors. This option can only be used\n");
  printf("  with option -C.\n");
  printf("- If outputfile is omitted, output is directed to screen.\n");
  printf("- If option -O is given, existing output file will be overwritten.\n");
  printf("- If -C option is used, parameter <outputfile> must be given.\n");
  printf("- Default codebook size is 256.\n");
  printf("- Pressing escape aborts program when the evaluation\n");
  printf("  of current codebook is finished.\n\n");
  printf("                                             Juha Kivijrvi  1995-96\n");
*/
  printf("\n%s %s          %s\n", ProgName, VersionNumber, LastUpdated);
  printf("This program generates random codebooks from given training\n");
  printf("set file and calculates the average error for each codebook.\n");
  printf("This program can also be used for generating a single random ");
  printf("codebook file.\n\n");
  printf("Usage: %s [%coption] <TSfile> [outputfile]\n", ProgName, OPTION_SYMBOL);
  printf("\n  Options:\n");
  PrintOptions();
  printf("\n                                             Juha Kivijrvi  1995-97\n");
}


/* ----------------------------------------------------------------- */

void OutputTime(time_t StartingTime)
{
time_t endtime;
long time_elapsed;
long h,m,s;


endtime=time(NULL);
time_elapsed=(long) difftime(endtime,StartingTime);

h=time_elapsed/3600;
m=(time_elapsed % 3600)/60;
s=time_elapsed%60;

printf("Time elapsed %ld:%ld%ld:%ld%ld\n",h,
    m/10, m%10, s/10, s%10);

}        




/*================  C O D E B O O K S  ==============================*/


int FreqToIndex(int freqsum, TRAININGSET* TS)

{
int i=0;
int l=0;

while (l<freqsum)
    {
    l += VectorFreq(TS,i);
    i++;
    }
return (i-1);
}


/*-------------------------------------------------------------------*/


void GetRandomVector(BOOKNODE* node, TRAININGSET* TS)
{
int t;

t=irand(1,TotalFreq(TS));
CopyNode( &Node(TS, FreqToIndex(t,TS)), node, VectorSize(TS) );
}


/*-------------------------------------------------------------------*/



void GenerateRandomCodebook(CODEBOOK* CB, TRAININGSET* TS)
{
int i,j;
YESNO Ok;

for (i=0; i<Value(CodebookSize); i++)
    {
    do
        {
        Ok = YES;
        GetRandomVector(&Node(CB,i), TS);
        if (!Value(AllowDuplicates))
	    for (j=0; j<i; j++)
		if (EqualVectors(Vector(CB,i), Vector(CB,j), VectorSize(CB)))
		    Ok=NO;
        }
    while (Ok==NO);    
    }
}


/*-------------------------------------------------------------------*/


void GenerateFullyRandomCodebook(CODEBOOK* CB)

{
int     i,j;
YESNO   Ok;

for (i=0; i<Value(CodebookSize); i++)
    {
    do
        {
        Ok=YES;
        for (j=0; j<VectorSize(CB); j++)
            VectorScalar(CB, i, j) = irand(0,CB->MaxValue);
        VectorFreq(CB, i) = 1;
	if (!Value(AllowDuplicates))
	    for (j=0; j<i; j++)
		if (EqualVectors(Vector(CB,i), Vector(CB,j), VectorSize(CB)))
		    Ok=NO;
        }
    while (Ok==NO);
    }

}



/*===========================  M A I N  ==============================*/



int main(int argc, char* argv[])
{
  TRAININGSET   TS;
  CODEBOOK      CB;
  char          TSName[80];
  char          OutName[80];
  FILE*         OutFile;
  time_t        StartingTime;
  YESNO         GoOn = YES;
  int           CBcount = 0;

  ParameterInfo paraminfo[2] =  { { TSName,  FormatNameTS, 0, INFILE   }, 
                                  { OutName, ""          , 1, OUTFILE  } };
                               
  ParseParameters( argc, argv, 2, paraminfo);

  StartingTime = time(NULL);

  initrandom(0);

  ReadTrainingSet(TSName, &TS);
  CreateNewCodebook(&CB, Value(CodebookSize), &TS);

  if ( Value(CreateCodebookfile) )
    {
    if ( strlen(OutName) < 1 )
        {
        printf("ERROR: codebook name not specified\n");
        exit(-1);
        }
    CheckFileName(OutName, FormatNameCB);
    if ( Value(UseWholeSpace) )
      GenerateFullyRandomCodebook(&CB);
    else
      GenerateRandomCodebook(&CB, &TS);
    printf("Error = %f\n",AverageErrorCB(&TS, &CB, MSE)); 
    WriteCodebook(OutName, &CB, Value(OverWrite));
    }
  else
    {
    if ( strlen(OutName) > 0 )
      OutFile = OpenFile(OutName, OUTPUT, Value(OverWrite));
    else
      OutFile = stdout;

    while (GoOn)
      {
      if ( Value(UseWholeSpace) )
        GenerateFullyRandomCodebook(&CB);
      else
        GenerateRandomCodebook(&CB, &TS);
      fprintf(OutFile,"%f\n",AverageErrorCB(&TS, &CB, MSE));
      #if defined(__MSDOS__)
      if (kbhit())
          GoOn=(getch()!=27); 
      #endif
      CBcount++;
      if (CBcount >= Value(CodebookCount) && Value(CodebookCount) > 0)
          GoOn=NO;
      }
    }

  FreeCodebook(&TS);
  FreeCodebook(&CB);

  checkmemory();
 
  if ( Value(ShowProgress) >= 1 )
    OutputTime(StartingTime);
 
  return( 0 );
}


